# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International
# are copyright Arch Systems Inc.

import unittest
import os.path
import os
import platform
from iotile.core.exceptions import *
from iotile.core.utilities.paths import settings_directory

class TestSettingsDirectory(unittest.TestCase):
    def setUp(self):
        self.settings_dir = settings_directory()
        self.confdir = os.path.join(os.path.expanduser('~'), '.config')

    @unittest.skipIf(platform.system() == 'Windows', 'Linux/Mac specific test')
    def test_settings_linix(self):
        settings_dir = os.path.abspath(os.path.join(self.confdir, 'IOTile-Core'))

        assert settings_dir == self.settings_dir

    @unittest.skipIf(platform.system() != 'Windows', 'Windows specific test')
    def test_settings_windows(self):
        if 'APPDATA' in os.environ:
            base = os.environ['APPDATA']
        else:
            base = self.confdir

        settings_dir = os.path.abspath(os.path.join(base, 'IOTile-Core'))
        assert settings_dir == self.settings_dir



import json

def convert(arg, **kwargs):
	if arg is None:
		return None

	if isinstance(arg, basestring):
		return json.loads(arg)
	elif isinstance(arg, dict):
		return arg
	
	raise TypeError("Unknown argument type")

def _json_formatter(arg):
	if isinstance(arg, bytearray):
		return repr(arg)

	return str(arg)
	
def default_formatter(arg, **kwargs):
	return json.dumps(arg, sort_keys=True, indent=4, separators=(',', ': '), default=_json_formatter)


from verifier import Verifier
from iotile.core.exceptions import ValidationError


class DictionaryVerifier(Verifier):
    """Verify the keys and values in a Dictionary

    Args:
        desc (string): A description of what this dictionary should contain
    """

    def __init__(self, desc=None):
        super(DictionaryVerifier, self).__init__(desc)

        self._required_keys = {}
        self._optional_keys = {}
        self._additional_key_rules = []

    def add_required(self, key, verifier):
        """Add a required key by name

        Args:
            key (string): The key name to match
            verifier (Verifier): The verification rule
        """

        self._required_keys[key] = verifier

    def add_optional(self, key, verifier):
        """Add an optional key by name

        Args:
            key (string): The key name to match
            verifier (Verifier): The verification rule
        """

        self._optional_keys[key] = verifier

    def verify(self, obj):
        """Verify that the object conforms to this verifier's schema

        Args:
            obj (object): A python object to verify

        Raises:
            ValidationError: If there is a problem verifying the dictionary, a
                ValidationError is thrown with at least the reason key set indicating
                the reason for the lack of validation.
        """

        out_obj = {}

        if not isinstance(obj, dict):
            raise ValidationError("Invalid dictionary", reason="object is not a dictionary")

        unmatched_keys = set(obj.keys())
        required_keys = set(self._required_keys.keys())

        # First check and make sure that all required keys are included and verify them
        for key in required_keys:
            if key not in unmatched_keys:
                raise ValidationError("Required key not found in dictionary", reason="required key %s not found" % key, key=key)

            out_obj[key] = self._required_keys[key].verify(obj[key])
            unmatched_keys.remove(key)

        # Now check and see if any of the keys in the dictionary are optional and check them
        to_remove = set()
        for key in unmatched_keys:
            if key not in self._optional_keys:
                continue

            out_obj[key] = self._optional_keys[key].verify(obj[key])
            to_remove.add(key)

        unmatched_keys -= to_remove

        # If there are additional keys, they need to match at least one of the additional key rules
        if len(unmatched_keys) > 0:
            if len(self._additional_key_rules) == 0:
                raise ValidationError("Extra key found in dictionary that does not allow extra keys", reason="extra keys found that were not expected", keys=unmatched_keys)

            to_remove = set()
            for key in unmatched_keys:
                for rule in self._additional_key_rules:
                    if rule[0].matches(key):
                        out_obj[key] = rule[1].verify(obj[key])
                        to_remove.add(key)
                        break

            unmatched_keys -= to_remove

            if len(unmatched_keys) > 0:
                raise ValidationError("Extra key found in dictionary that did not match any extra key rule", reason="extra keys found that did not match any rule", keys=unmatched_keys)

        return out_obj



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

#basic tests to ensure that everything is installed and working properly

def test_required_packages():
	"""
	Make sure that the required dependencies of iotile.core are installed
	"""

	import decorator
	#TODO add the rest of the packages

def test_import_iotilecore():
	"""
	Make sure iotile.core is accessible for import
	"""

	import iotile.core


"""A class that dispatches messages to handlers based on their schema."""
from iotile.core.exceptions import ArgumentError


class ValidatingDispatcher(object):
    """An object that dispatches messages to handlers based on their schema."""

    def __init__(self):
        self.validators = []

    def add_message_type(self, validator, callback):
        """Add a message type that should trigger a callback.

        Each validator must be unique, in that a message will
        be dispatched to the first callback whose validator
        matches the message.

        Args:
            validator (Verifier): A schema verifier that will
                validate a received message uniquely
            callback (callable): The function that should be called
                when a message that matches validator is received.
        """

        self.validators.append((validator, callback))

    def dispatch(self, message):
        """Dispatch a message to a callback based on its schema.

        Args:
            message (dict): The message to dispatch
        """

        for validator, callback in self.validators:
            if not validator.matches(message):
                continue

            callback(message)
            return

        raise ArgumentError("No handler was registered for message", message=message)



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

#rcfile.py
#A simple way to store and access configuration files in a cross-platform way.
#It uses dirspec to find the appropriate config directory for each platform and
#then creates a file under that directory with the appropriate application name

import os.path
from paths import settings_directory
from iotile.core.exceptions import *

class RCFile:
	"""
	A simple object that represents a configuration file stored in a cross-platform
	way by looking for the appropriate configuration directory on each platform.  The 
	file is named NAME_config.txt and its contents are accessible via the .contents
	attribute of the class.
	"""

	def __init__(self, name):
		"""
		Access the configuration file for the configurable object name.  Multiple calls
		with the same name string will return the same configuration file.
		"""

		self.name = name
		self.path = self._build_path()
		self.valid= True
		self.error = ""

		#Read in the file ignoring all blank lines and chopping off newline characters
		try:
			with open(self.path, "r") as f:
				contents = f.readlines()

			self.contents = [x.rstrip() for x in contents if len(x.rstrip()) > 0]
		except IOError as e:
			self.valid = False
			self.error = str(e)
			self.contents = []

	def _build_path(self):		
		fname = "%s_config.txt" % self.name

		return os.path.join(settings_directory(), fname)

	def save(self):
		"""
		Update the configuration file on disk with the current contents of self.contents.
		Previous contents are overwritten.
		"""

		try:
			with open(self.path, "w") as f:
				f.writelines(self.contents)
		except IOError as e:
			raise InternalError("Could not write RCFile contents", name=self.name, path=self.path, error_message=str(e))



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

from iotile.core.hw.proxy.proxy import TileBusProxyObject

class EmptyProxy (TileBusProxyObject):
	def random_method(self):
		return True



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

#list.py

class list(object):
	def __init__(self, valuetype, **kwargs):
		
		self.valuetype = valuetype
		self.type_system = kwargs['type_system']

	@staticmethod
	def Build(*types, **kwargs):
		if len(types) != 1:
			raise ValueError("list must be created with 1 argument, a value type")
		
		return list(types[0], **kwargs)

	def convert(self, value, **kwargs):
		converted = []
		for x in value:
			y = self.type_system.convert_to_type(x, self.valuetype, **kwargs)
			converted.append(y)

		return converted

	def default_formatter(self, value, **kwargs):
		lines = []
		for x in value:
			line = self.type_system.format_value(x, self.valuetype, **kwargs)
			lines.append(line)

		return "\n".join(lines)

	def format_compact(self, value, **kwargs):
		lines = []
		for x in value:
			line = self.type_system.format_value(x, self.valuetype, **kwargs)
			lines.append(line)

		return "[" + ", ".join(lines) + "]"


"""A VirtualDevice composed of various tiles.

The device handles RPCs by dispatching them to tiles configured using a config dictionary.
"""

from .virtualdevice import VirtualIOTileDevice
from .virtualtile import VirtualTile


class TileBasedVirtualDevice(VirtualIOTileDevice):
    """A VirtualDevice composed of one or more tiles.

    Args:
        args (dict): A dictionary that lists the tiles that
            should be loaded to create this virtual device.
    """

    def __init__(self, args):
        iotile_id = args.get('iotile_id')

        if isinstance(iotile_id, (basestring, unicode)):
            iotile_id = int(iotile_id, 16)

        tiles = args.get('tiles', [])
        name = args.get('name', "No Name")

        super(TileBasedVirtualDevice, self).__init__(iotile_id, name)

        for desc in tiles:
            name = desc['name']
            address = desc['address']

            args = desc.get('args', {})

            tile_type = VirtualTile.FindByName(name)
            tile = tile_type(address, args)

            self.add_tile(address, tile)

    def start(self, channel):
        """Start running this virtual device including any necessary worker threads.

        Args:
            channel (IOTilePushChannel): the channel with a stream and trace
                routine for streaming and tracing data through a VirtualInterface
        """

        super(TileBasedVirtualDevice, self).start(channel)

        for tile in self._tiles.itervalues():
            tile.start()

    def stop(self):
        """Stop running this virtual device including any worker threads."""

        for tile in self._tiles.itervalues():
            tile.signal_stop()

        for tile in self._tiles.itervalues():
            tile.wait_stopped()

        super(TileBasedVirtualDevice, self).stop()



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International 
# are copyright Arch Systems Inc.

import pytest
from iotile.core.utilities.typedargs import type_system
from iotile.core.utilities import typedargs
from iotile.core.exceptions import *

def test_builtins_exist():
	builtin = ['integer', 'path', 'string']

	for b in builtin:
		type_system.get_type(b)

def test_builtin_conversions():
	val = type_system.convert_to_type('42', 'integer')
	assert val == 42

	val = type_system.convert_to_type('/users/timburke', 'path')
	assert val == '/users/timburke'

	val = type_system.convert_to_type('hello', 'string')
	assert val == 'hello'

def test_annotation_correct():
	@typedargs.param("string_param", "string", desc='Hello')
	def function_test(string_param):
		pass

	function_test("hello")

def test_annotation_unknown_type():
	with pytest.raises(ArgumentError):
		@typedargs.param("string_param", "unknown_type", desc='Hello')
		def function_test(string_param):
			pass

		function_test("hello")

def test_annotation_validation():
	with pytest.raises(ValidationError):
		@typedargs.param("int_param", "integer", "nonnegative", desc="No desc")
		def function_test(int_param):
			pass

		function_test(-1)

def test_bool_valid():
	val = type_system.convert_to_type('True', 'bool')
	assert val == True

	val = type_system.convert_to_type('false', 'bool')
	assert val == False

	val = type_system.convert_to_type(True, 'bool')
	assert val == True

	val = type_system.convert_to_type(False, 'bool')
	assert val == False

	val = type_system.convert_to_type(None, 'bool')
	assert val is None

	val = type_system.convert_to_type(0, 'bool')
	assert val == False
	
	val = type_system.convert_to_type(1, 'bool')
	assert val == True

def test_format_bool():
	val = type_system.format_value(True, 'bool')
	assert val == 'True'

	val = type_system.format_value(False, 'bool')
	assert val == 'False'



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International
# are copyright Arch Systems Inc.

from iotile.core.hw.exceptions import *
from iotile.core.exceptions import *
from iotile.core.hw.commands import RPCCommand
import atexit
import json
import binascii

open_streams = set()

def do_final_close():
    """
    Make sure that all streams are properly closed at shutdown
    """

    #Make a copy since stream.close will remove the stream from the master set
    streams = open_streams.copy()
    for stream in streams:
        stream.close()

atexit.register(do_final_close)

class CMDStream(object):
    """
    Any physical method that supports talking to an IOTile based device

    All interactions with the IOTile device will be via one of the primitive operations defined in this
    class. Specific implementations may transfer the data in their own way and add additional layers
    as needed. Examples of CMDStream implementations are:
    - the Field Service Unit communicating over a USB <-> Serial bridge
    - Bluetooth LE (directly)
    - Bluetooth LE by way of the RN4020 module connected to a USB port of the com
    """

    def __init__(self, port, connection_string, record=None):
        self.connection_string = connection_string
        self.connected = False
        self.port = port
        self.record = record
        self.opened = True

        open_streams.add(self)

        if self.record is not None:
            self._recording = {}

        if self.connection_string != None:
            try:
                self.connect_direct(self.connection_string)
            except:
                self.close()
                raise

    def scan(self, wait=None):
        """Scan for available IOTile devices.

        Scan for connected device and return a map of UUIDs and connection strings for
        all of the devices that were found.

        Args:
            wait (float): Optional amount of time to force the device adapter to wait before
                returning.
        """

        if not hasattr(self, '_scan'):
            raise StreamOperationNotSupportedError(command="scan")

        return sorted(self._scan(wait=wait), key=lambda x: x['uuid'])

    def connect_direct(self, connection_string):
        """Directly connect to a device using its stream specific connection string
        """

        if self.connected:
            raise HardwareError("Cannot connect when we are already connected")

        if not hasattr(self, '_connect_direct'):
            raise StreamOperationNotSupportedError(command="connect_direct")

        self._connect_direct(connection_string)
        self.connected = True
        self.connection_string = connection_string

    def connect(self, uuid_value, wait=None):
        """Connect to a specific device by its uuid

        Attempt to connect to a device that we have previously scanned using its UUID.
        If wait is not None, then it is used in the same was a scan(wait) to override
        default wait times with an explicit value.

        Args:
            wait (float): Optional amount of time to force the device adapter to wait before
                atttempting to connect.
        """

        if self.connected:
            raise HardwareError("Cannot connect when we are already connected")

        if not hasattr(self, '_connect'):
            raise StreamOperationNotSupportedError(command="connect")

        connection_string = self._connect(uuid_value, wait=wait)

        self.connected = True
        self.connection_string = connection_string

    def disconnect(self):
        """Disconnect from the device that we are currently connected to
        """
        if not self.connected:
            raise HardwareError("Cannot disconnect when we are not connected")

        if not hasattr(self, '_disconnect'):
            raise StreamOperationNotSupportedError(command="disconnect")

        self._disconnect()
        self.connected = False

    def send_rpc(self, address, feature, command, *args, **kwargs):
        if not self.connected:
            raise HardwareError("Cannot send an RPC if we are not in a connected state")

        if not hasattr(self, '_send_rpc'):
            raise StreamOperationNotSupportedError(command="send_rpc")

        rpc = RPCCommand(address, feature, command, *args)
        call_payload = rpc._format_args()
        call_payload = call_payload[:rpc.spec]

        status, payload = self._send_rpc(address, feature, command, call_payload, **kwargs)

        #If we are recording this, save off the call and response
        if self.record is not None:
            if self.connection_string not in self._recording:
                self._recording[self.connection_string] = []

            call = "{0},{1},{2},{3}".format(address, feature, command, binascii.hexlify(call_payload))
            response = "{0},{1}".format(status, binascii.hexlify(payload))

            self._recording[self.connection_string].append((call, response))

        if status == 0:
            raise ModuleBusyError(address)
        elif status == 0xFF:
            raise ModuleNotFoundError(address)

        return status, bytearray(payload)

    def enable_streaming(self):
        if not self.connected:
            raise HardwareError("Cannot enable streaming if we are not in a connected state")

        if not hasattr(self, '_enable_streaming'):
            raise StreamOperationNotSupportedError(command="enable_streaming")

        return self._enable_streaming()

    def enable_tracing(self):
        if not self.connected:
            raise HardwareError("Cannot enable tracing if we are not in a connected state")

        if not hasattr(self, '_enable_tracing'):
            raise StreamOperationNotSupportedError(command="enable_tracing")

        return self._enable_tracing()

    def send_highspeed(self, data, progress_callback=None):
        if not self.connected:
            raise HardwareError("Cannot send highspeed data if we are not in a connected state")

        if not hasattr(self, '_send_highspeed'):
            raise StreamOperationNotSupportedError(command="send_highspeed")

        return self._send_highspeed(data, progress_callback)

    def heartbeat(self):
        if not hasattr(self, '_heartbeat'):
            raise StreamOperationNotSupportedError(command="heartbeat")

        return self._heartbeat()

    def reset(self):
        if not hasattr(self, '_reset'):
            raise StreamOperationNotSupportedError(command="reset")

        self._reset()

    def close(self):
        if not self.opened:
            print("close called twice on the same stream")
            return

        # Do not raise an error if no internal _close routine is found, that
        # just means that no stream specific close operations are required
        try:
            if hasattr(self, '_close'):
                self._close()
        finally:
            #Make sure that no matter what happens we save this recording out
            self._save_recording()
            self.opened = False
            open_streams.remove(self)

    def _save_recording(self):
        if not self.record:
            return

        with open(self.record, "w") as f:
            json.dump(self._recording, f, indent=4)

        self.record = False



# This file is copyright Arch Systems, Inc.  
# Except as otherwise provided in the relevant LICENSE file, all rights are reserved.

#bytes.py
#Simple bytearray type

def convert(arg, **kwargs):
	if isinstance(arg, bytearray):
		return arg
	elif isinstance(arg, basestring):
		if len(arg) > 2 and arg.startswith("0x"):
			data = arg[2:].decode('hex')
		else:
			data = arg
		return bytearray(data)

	raise TypeError("You must create a bytes object from a bytearray or a hex string")

def convert_binary(arg, **kwargs):
	return bytearray(arg)

def default_formatter(arg, **kwargs):
	return str(arg)

def format_repr(arg):
	return repr(arg)


# This file is copyright Arch Systems, Inc.  
# Except as otherwise provided in the relevant LICENSE file, all rights are reserved.

#Sqlite3 textual key value store
from iotile.core.utilities.paths import settings_directory
import sqlite3
import os.path
import sys
import os

class SQLiteKVStore(object):
    """
    A simple string - string persistent map backed by sqlite for concurrent access

    The KeyValueStore can be made to respect python virtual environments if desired
    """

    DefaultFolder = settings_directory()

    def __init__(self, name, folder=None, respect_venv=False):
        if folder is None:
            folder = SQLiteKVStore.DefaultFolder

        #If we are relative to a virtual environment, place the registry into that virtual env
        #Support both virtualenv and pythnon 3 venv
        if respect_venv and hasattr(sys, 'real_prefix'):
            folder = sys.prefix
        elif respect_venv and hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
            folder = sys.prefix

        if not os.path.exists(folder):
            os.makedirs(folder, 0755)

        dbfile = os.path.join(folder, name)
        self.connection = sqlite3.connect(dbfile)
        self.cursor = self.connection.cursor()

        self.file = dbfile
        self._setup_table()

    def _setup_table(self):
        query = 'create table if not exists KVStore (key TEXT PRIMARY KEY, value TEXT);'
        self.cursor.execute(query)
        self.connection.commit()

    def size(self):
        query = 'select count(*) from KVStore'
        self.cursor.execute(query)
        return self.cursor.fetchone()[0]

    def get_all(self):
        query = 'select key, value from KVStore'
        self.cursor.execute(query)

        vals = self.cursor.fetchall()
        return vals
        
    def get(self, id):
        query = 'select value from KVStore where key is ?'
        self.cursor.execute(query, (id,))

        val = self.cursor.fetchone()
        if val is None:
            raise KeyError("id not in key-value store: %s" % str(id))
        
        return val[0]

    def remove(self, key):
        query = "delete from KVStore where key is ?"
        self.cursor.execute(query, (key,))
        self.connection.commit()

    def try_get(self, id):
        try:
            return self.get(id)
        except KeyError:
            return None

    def set(self, key, value):
        query = "insert or replace into KVStore values (?, ?)"
        self.cursor.execute(query, (key, str(value)))
        self.connection.commit()

    def clear(self):
        query = 'drop table KVStore'
        self.cursor.execute(query)
        self.connection.commit()

        self._setup_table()



from .dict_verify import DictionaryVerifier
from .list_verify import ListVerifier
from .string_verify import StringVerifier
from .int_verify import IntVerifier
from .float_verify import FloatVerifier
from .bool_verify import BooleanVerifier
from .literal_verify import LiteralVerifier
from .options_verify import OptionsVerifier
from .enum_verify import EnumVerifier
from .bytes_verify import BytesVerifier

from .verifier import Verifier

__all__ = ['DictionaryVerifier', 'ListVerifier', 'StringVerifier', 'IntVerifier', 'FloatVerifier',
           'BooleanVerifier', 'LiteralVerifier', 'OptionsVerifier', 'EnumVerifier', 'Verifier',
           'BytesVerifier']



# This file is adapted from python code released by WellDone International
# under the terms of the LGPLv3.  WellDone International's contact information is
# info@welldone.org
# http://welldone.org
#
# Modifications to this file from the original created at WellDone International
# are copyright Arch Systems Inc.

from iotile.core.hw.hwmanager import HardwareManager
from iotile.core.hw.reports.signed_list_format import SignedListReport
from iotile.core.hw.exceptions import *
from iotile.core.exceptions import *
import pytest
import os.path
import os
import time


@pytest.fixture
def simple_hw():
    hw = HardwareManager('virtual:simple')
    yield hw

    hw.disconnect()


@pytest.fixture
def report_hw():
    hw = HardwareManager('virtual:report_test')
    yield hw

    hw.disconnect()


@pytest.fixture
def conf_report_hw():
    conf_file = os.path.join(os.path.dirname(__file__), 'report_test_config_hash.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:report_test@%s' % conf_file)
    yield hw

    hw.disconnect()


@pytest.fixture
def realtime_hw():
    conf_file = os.path.join(os.path.dirname(__file__), 'fast_realtime_test.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:realtime_test@%s' % conf_file)
    yield hw

    hw.disconnect()

@pytest.fixture
def realtime_scan_hw():
    conf_file = os.path.join(os.path.dirname(__file__), 'fast_realtime_test.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:realtime_test@%s' % conf_file)
    yield hw

    hw.disconnect()

@pytest.fixture
def tile_based():
    conf_file = os.path.join(os.path.dirname(__file__), 'tile_config.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:tile_based@%s' % conf_file)
    yield hw

    hw.disconnect()

@pytest.fixture
def tracer_hw():
    conf_file = os.path.join(os.path.dirname(__file__), 'fast_realtime_trace.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:realtime_test@%s' % conf_file)
    yield hw

    hw.disconnect()


@pytest.fixture
def conf2_report_hw():
    conf_file = os.path.join(os.path.dirname(__file__), 'report_test_config_signed.json')

    if '@' in conf_file or ',' in conf_file or ';' in conf_file:
        pytest.skip('Cannot pass device config because path has [@,;] in it')

    hw = HardwareManager('virtual:report_test@%s' % conf_file)
    yield hw

    hw.disconnect()

def test_basic(simple_hw):
    simple_hw.connect_direct('1')


def test_report(report_hw):
    report_hw.connect_direct('1')
    report_hw.enable_streaming()
    assert report_hw.count_reports() == 100


def test_config_file(conf_report_hw):
    """Make sure we can pass a config dict
    """

    conf_report_hw.connect_direct('2')
    conf_report_hw.enable_streaming()
    assert conf_report_hw.count_reports() == 11


def test_config_file2(conf2_report_hw, monkeypatch):
    """Make sure we can sign reports
    """

    monkeypatch.setenv('USER_KEY_00000002', '0000000000000000000000000000000000000000000000000000000000000000')

    conf2_report_hw.connect_direct('2')
    conf2_report_hw.enable_streaming()

    assert conf2_report_hw.count_reports() == 11

    for report in conf2_report_hw.iter_reports():
        assert report.verified
        assert report.signature_flags == 1
        assert report.lowest_id >= 1
        assert report.highest_id > report.lowest_id
        assert isinstance(report, SignedListReport)


def test_realtime_streaming(realtime_hw):
    """Make sure we properly support streaming asynchronously
    """

    realtime_hw.connect_direct('1')
    realtime_hw.enable_streaming()

    reports = realtime_hw.wait_reports(10)

    stream1 = [x for x in reports if x.visible_readings[0].stream == 0x100a]
    stream2 = [x for x in reports if x.visible_readings[0].stream == 0x5001]

    assert len(stream1) != 0
    assert len(stream2) != 0

    assert stream1[0].visible_readings[0].value == 200
    assert stream2[0].visible_readings[0].value == 100


def test_realtime_tracing(tracer_hw):
    """Make sure we properly support tracing data asynchronously
    """

    tracer_hw.connect_direct('1')
    tracer_hw.enable_tracing()

    time.sleep(.1)

    trace_data = tracer_hw.dump_trace('raw')

    assert len(trace_data) > 0
    words = trace_data.split(' ')

    wrong_data = [x for x in words if (x != 'hello' and x != 'goodbye' and x != '')]
    assert len(wrong_data) == 0


def test_virtual_scan(realtime_scan_hw):
    """Make sure we can scan for virtual devices and connect directly without connect_direct
    """
    devices = realtime_scan_hw.scan()

    assert len(devices) > 0
    realtime_scan_hw.connect('1')


def test_tilebased_device(tile_based):
    """Make sure we can interact with tile based virtual devices."""

    hw = tile_based

    hw.connect(1)
    con = hw.controller()
    tile1 = hw.get(11)

    assert con.add(1, 2) == 3
    con.count()

    assert tile1.add(3, 5) == 8
    tile1.count()
